@@ -895,6 +895,8 @@ void parse_db_table_from_file_path(
 		*ptr = '\0';
 	if ((ptr = strstr(tablename, "#P#")))
 		*ptr = '\0';
+	if ((ptr = strstr(tablename, "#i#")))
+		*ptr = '\0';
 }
 
 bool is_system_table(const char *dbname, const char *tablename)
@@ -2130,7 +2132,7 @@ ds_ctxt_t::make_hardlink(const char *from_path, const char *to_path)
 	}
 	else
 	{
-		strncpy(to_path_full, to_path, sizeof(to_path_full)-1);
+		strmake(to_path_full, to_path, sizeof(to_path_full)-1);
 	}
 #ifdef _WIN32
 	return  CreateHardLink(to_path_full, from_path, NULL);
