@@ -897,3 +897,14 @@ DELIMITER ;
 ALTER TABLE servers
   MODIFY Host varchar(2048) NOT NULL DEFAULT '',
   MODIFY Owner varchar(512) NOT NULL DEFAULT '';
+
+# MDEV-34716 Fix mysql.servers socket max length too short
+ALTER TABLE servers
+  MODIFY Socket char(108) NOT NULL DEFAULT '';
+
+# MDEV-34716 Allow arbitrary options in CREATE SERVER
+ALTER TABLE servers
+  ADD Options JSON NOT NULL DEFAULT '{}' CHECK(JSON_VALID(Options));
+# Ensure the collation is utf8mb4_bin (default for JSON)
+ALTER TABLE servers
+  MODIFY Options JSON  NOT NULL DEFAULT '{}' CHECK(JSON_VALID(Options));
