@@ -21,10 +21,6 @@
   The actual communction is handled by the net_xxx functions in net_serv.cc
 */
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation				// gcc: Class implementation
-#endif
-
 #include "mariadb.h"
 #include "sql_priv.h"
 #include "protocol.h"
@@ -593,6 +589,7 @@ void Protocol::end_statement()
 
   switch (thd->get_stmt_da()->status()) {
   case Diagnostics_area::DA_ERROR:
+    thd->stop_collecting_unit_results();
     /* The query failed, send error to log and abort bootstrap. */
     error= send_error(thd->get_stmt_da()->sql_errno(),
                       thd->get_stmt_da()->message(),
@@ -600,12 +597,36 @@ void Protocol::end_statement()
     break;
   case Diagnostics_area::DA_EOF:
   case Diagnostics_area::DA_EOF_BULK:
-    error= send_eof(thd->server_status,
+    if (thd->need_report_unit_results()) {
+      // bulk returning result-set, like INSERT ... RETURNING
+      // result is already send, needs an EOF with MORE_RESULT_EXISTS
+      // before sending unit result-set
+      error= send_eof(thd->server_status | SERVER_MORE_RESULTS_EXISTS,
+                          thd->get_stmt_da()->statement_warn_count());
+      if (thd->report_collected_unit_results() && thd->is_error())
+        error= send_error(thd->get_stmt_da()->sql_errno(),
+                          thd->get_stmt_da()->message(),
+                          thd->get_stmt_da()->get_sqlstate());
+      else
+        error= send_eof(thd->server_status,
+                    thd->get_stmt_da()->statement_warn_count());
+    }
+    else
+      error= send_eof(thd->server_status,
                     thd->get_stmt_da()->statement_warn_count());
     break;
   case Diagnostics_area::DA_OK:
   case Diagnostics_area::DA_OK_BULK:
-    error= send_ok(thd->server_status,
+    if (thd->report_collected_unit_results())
+      if (thd->is_error())
+        error= send_error(thd->get_stmt_da()->sql_errno(),
+                        thd->get_stmt_da()->message(),
+                        thd->get_stmt_da()->get_sqlstate());
+      else
+        error= send_eof(thd->server_status,
+                     thd->get_stmt_da()->statement_warn_count());
+    else
+      error= send_ok(thd->server_status,
                    thd->get_stmt_da()->statement_warn_count(),
                    thd->get_stmt_da()->affected_rows(),
                    thd->get_stmt_da()->last_insert_id(),
@@ -615,6 +636,7 @@ void Protocol::end_statement()
     break;
   case Diagnostics_area::DA_EMPTY:
   default:
+    thd->stop_collecting_unit_results();
     DBUG_ASSERT(0);
     error= send_ok(thd->server_status, 0, 0, 0, NULL);
     break;
@@ -877,13 +899,17 @@ bool Protocol_text::store_field_metadata(const THD * thd,
     if (charset_for_protocol == &my_charset_bin || thd_charset == NULL)
     {
       /* No conversion */
-      int2store(pos, charset_for_protocol->number);
+      uint id= charset_for_protocol->get_id(MY_COLLATION_ID_TYPE_COMPAT_100800);
+      DBUG_ASSERT(id <= UINT_MAX16);
+      int2store(pos, (uint16) id);
       int4store(pos + 2, field.length);
     }
     else
     {
       /* With conversion */
-      int2store(pos, thd_charset->number);
+      uint id= thd_charset->get_id(MY_COLLATION_ID_TYPE_COMPAT_100800);
+      DBUG_ASSERT(id <= UINT_MAX16);
+      int2store(pos, (uint16) id);
       uint32 field_length= field.max_octet_length(charset_for_protocol,
                                                   thd_charset);
       int4store(pos + 2, field_length);
@@ -1200,8 +1226,7 @@ bool Protocol::send_result_set_metadata(List<Item> *list, uint flags)
     Item *item;
     Protocol_text prot(thd, thd->variables.net_buffer_length);
 #ifndef DBUG_OFF
-    field_handlers= (const Type_handler **) thd->alloc(
-        sizeof(field_handlers[0]) * list->elements);
+    field_handlers= thd->alloc<const Type_handler *>(list->elements);
 #endif
 
     for (uint pos= 0; (item= it++); pos++)
@@ -1250,8 +1275,7 @@ bool Protocol::send_list_fields(List<Field> *list, const TABLE_LIST *table_list)
   Protocol_text prot(thd, thd->variables.net_buffer_length);
 
 #ifndef DBUG_OFF
-  field_handlers= (const Type_handler **) thd->alloc(sizeof(field_handlers[0]) *
-                                                     list->elements);
+  field_handlers= thd->alloc<const Type_handler *>(list->elements);
 #endif
 
   for (uint pos= 0; (fld= it++); pos++)
@@ -1371,19 +1395,20 @@ bool Protocol::store(I_List<i_string>* str_list)
 {
   char buf[256];
   String tmp(buf, sizeof(buf), &my_charset_bin);
-  uint32 len;
+  size_t len= 0;
   I_List_iterator<i_string> it(*str_list);
   i_string* s;
+  const char *delimiter= ",";
 
   tmp.length(0);
   while ((s=it++))
   {
+    tmp.append(delimiter, len);
     tmp.append(s->ptr, strlen(s->ptr));
-    tmp.append(',');
+    len= 1;
   }
-  if ((len= tmp.length()))
-    len--;					// Remove last ','
-  return store((char*) tmp.ptr(), len,  tmp.charset());
+
+  return store((char*) tmp.ptr(), tmp.length(),  tmp.charset());
 }
 
 /****************************************************************************
