@@ -556,6 +556,13 @@ typedef struct st_join_table {
   /** HAVING condition for checking prior saving a record into tmp table*/
   Item *having;
 
+  /**
+    Ordering to be produced when doing full index scan.
+    Important for vector indexes, set by test_if_skip_sort_order() when it
+    decides to use full index to produce rows in order.
+  */
+  ORDER *full_index_scan_order;
+
   /** TRUE <=> remove duplicates on this table. */
   bool distinct;
 
@@ -1440,6 +1447,7 @@ public:
 
   Pushdown_query *pushdown_query;
   JOIN_TAB *original_join_tab;
+  uint	   original_table_count;
   uint	   sort_space;
 
 /******* Join optimization state members start *******/
@@ -2023,7 +2031,7 @@ int opt_sum_query(THD* thd,
                   List<TABLE_LIST> &tables, List<Item> &all_fields, COND *conds);
 
 /* from sql_delete.cc, used by opt_range.cc */
-extern "C" int refpos_order_cmp(void* arg, const void *a,const void *b);
+extern "C" int refpos_order_cmp(void *arg, const void *a,const void *b);
 
 /** class to copying an field/item to a key struct */
 
@@ -2714,15 +2722,7 @@ void propagate_new_equalities(THD *thd, Item *cond,
                               COND_EQUAL *inherited,
                               bool *is_simplifiable_cond);
 
-template<typename T> T prev_bits(T n_bits)
-{
-  if (!n_bits)
-    return 0;
-  T tmp= ((T)1 << (n_bits - 1));
-  return (tmp - 1) | tmp;
-}
-// A wrapper for the above function:
-#define PREV_BITS(type, A) prev_bits<type>(A)
+#define PREV_BITS(type, N_BITS) ((type)my_set_bits(N_BITS))
 
 bool dbug_user_var_equals_str(THD *thd, const char *name, const char *value);
 
