@@ -291,7 +291,7 @@ public:
   uchar *tref() const;
   void push_neighbor(size_t layer, FVectorNode *v);
 
-  static uchar *get_key(const FVectorNode *elem, size_t *key_len, my_bool);
+  static const uchar *get_key(const void *elem, size_t *key_len, my_bool);
 };
 #pragma pack(pop)
 
@@ -686,8 +686,13 @@ int MHNSW_Share::acquire(MHNSW_Share **ctx, TABLE *table, bool for_update)
 
   graph->file->position(graph->record[0]);
   (*ctx)->set_lengths(FVector::data_to_value_size(graph->field[FIELD_VEC]->value_length()));
-  (*ctx)->start= (*ctx)->get_node(graph->file->ref);
-  return (*ctx)->start->load_from_record(graph);
+
+  auto node= (*ctx)->get_node(graph->file->ref);
+  if ((err= node->load_from_record(graph)))
+    return err;
+
+  (*ctx)->start= node; // set the shared start only when node is fully loaded
+  return 0;
 }
 
 /* copy the vector, preprocessed as needed */
@@ -812,10 +817,10 @@ size_t FVectorNode::gref_len() const { return ctx->gref_len; }
 uchar *FVectorNode::gref() const { return (uchar*)(this+1); }
 uchar *FVectorNode::tref() const { return gref() + gref_len(); }
 
-uchar *FVectorNode::get_key(const FVectorNode *elem, size_t *key_len, my_bool)
+const uchar *FVectorNode::get_key(const void *elem, size_t *key_len, my_bool)
 {
-  *key_len= elem->gref_len();
-  return elem->gref();
+  *key_len= static_cast<const FVectorNode*>(elem)->gref_len();
+  return static_cast<const FVectorNode*>(elem)->gref();
 }
 
 /* one visited node during the search. caches the distance to target */
@@ -824,8 +829,10 @@ struct Visited : public Sql_alloc
   FVectorNode *node;
   const float distance_to_target;
   Visited(FVectorNode *n, float d) : node(n), distance_to_target(d) {}
-  static int cmp(void *, const Visited* a, const Visited *b)
+  static int cmp(void *, const void* a_, const void *b_)
   {
+    const Visited *a= static_cast<const Visited*>(a_);
+    const Visited *b= static_cast<const Visited*>(b_);
     return a->distance_to_target < b->distance_to_target ? -1 :
            a->distance_to_target > b->distance_to_target ?  1 : 0;
   }
@@ -908,7 +915,7 @@ static int select_neighbors(MHNSW_Share *ctx, TABLE *graph, size_t layer,
   {
     Visited *vec= pq.pop();
     FVectorNode * const node= vec->node;
-    const float target_dista= vec->distance_to_target / alpha;
+    const float target_dista= std::max(32*FLT_EPSILON, vec->distance_to_target / alpha);
     bool discard= false;
     for (size_t i=0; i < neighbors.num; i++)
       if ((discard= node->distance_to(neighbors.links[i]->vec) <= target_dista))
@@ -1090,7 +1097,7 @@ static int search_layer(MHNSW_Share *ctx, TABLE *graph, const FVector *target,
         if (!best.is_full())
         {
           max_distance= std::max(max_distance, v->distance_to_target);
-          candidates.push(v);
+          candidates.safe_push(v);
           if (skip_deleted && v->node->deleted)
             continue;
           best.push(v);
@@ -1348,7 +1355,7 @@ int mhnsw_read_next(TABLE *table)
     }
     ctx->release(false, table->s);      // release shared ctx
     result->ctx= trx;                   // replace it with trx
-    result->ctx_version= trx->version; 
+    result->ctx_version= trx->version;
     std::swap(trx, ctx);        // free shared ctx in this scope, keep trx
   }
 
@@ -1358,7 +1365,7 @@ int mhnsw_read_next(TABLE *table)
                    static_cast<uint>(result->pos), 0, &result->found, false))
     return err;
   result->pos= 0;
-  result->threshold= new_threshold;
+  result->threshold= new_threshold + FLT_EPSILON;
   return mhnsw_read_next(table);
 }
 
