@@ -2,7 +2,7 @@
 
 Copyright (c) 2010, 2016, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2012, Facebook Inc.
-Copyright (c) 2013, 2022, MariaDB Corporation.
+Copyright (c) 2013, 2023, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -27,7 +27,6 @@ Created 12/9/2009 Jimmy Yang
 
 #include "buf0flu.h"
 #include "dict0mem.h"
-#include "ibuf0ibuf.h"
 #include "lock0lock.h"
 #include "mach0data.h"
 #include "os0file.h"
@@ -35,7 +34,6 @@ Created 12/9/2009 Jimmy Yang
 #include "srv0srv.h"
 #include "trx0rseg.h"
 #include "trx0sys.h"
-#include "log.h"
 
 /* Macro to standardize the counter names for counters in the
 "monitor_buf_page" module as they have very structured defines */
@@ -518,23 +516,10 @@ static monitor_info_t	innodb_counter_info[] =
 	MONITOR_BUF_PAGE_READ("index_non_leaf","Index Non-leaf",
 			      INDEX_NON_LEAF),
 
-	MONITOR_BUF_PAGE_READ("index_ibuf_leaf", "Insert Buffer Index Leaf",
-			      INDEX_IBUF_LEAF),
-
-	MONITOR_BUF_PAGE_READ("index_ibuf_non_leaf",
-			      "Insert Buffer Index Non-Leaf",
-			       INDEX_IBUF_NON_LEAF),
-
 	MONITOR_BUF_PAGE_READ("undo_log", "Undo Log", UNDO_LOG),
 
 	MONITOR_BUF_PAGE_READ("index_inode", "Index Inode", INODE),
 
-	MONITOR_BUF_PAGE_READ("ibuf_free_list", "Insert Buffer Free List",
-			      IBUF_FREELIST),
-
-	MONITOR_BUF_PAGE_READ("ibuf_bitmap", "Insert Buffer Bitmap",
-			      IBUF_BITMAP),
-
 	MONITOR_BUF_PAGE_READ("system_page", "System", SYSTEM),
 
 	MONITOR_BUF_PAGE_READ("trx_system", "Transaction System", TRX_SYSTEM),
@@ -557,23 +542,10 @@ static monitor_info_t	innodb_counter_info[] =
 	MONITOR_BUF_PAGE_WRITTEN("index_non_leaf","Index Non-leaf",
 				 INDEX_NON_LEAF),
 
-	MONITOR_BUF_PAGE_WRITTEN("index_ibuf_leaf", "Insert Buffer Index Leaf",
-				 INDEX_IBUF_LEAF),
-
-	MONITOR_BUF_PAGE_WRITTEN("index_ibuf_non_leaf",
-				 "Insert Buffer Index Non-Leaf",
-				 INDEX_IBUF_NON_LEAF),
-
 	MONITOR_BUF_PAGE_WRITTEN("undo_log", "Undo Log", UNDO_LOG),
 
 	MONITOR_BUF_PAGE_WRITTEN("index_inode", "Index Inode", INODE),
 
-	MONITOR_BUF_PAGE_WRITTEN("ibuf_free_list", "Insert Buffer Free List",
-				 IBUF_FREELIST),
-
-	MONITOR_BUF_PAGE_WRITTEN("ibuf_bitmap", "Insert Buffer Bitmap",
-				 IBUF_BITMAP),
-
 	MONITOR_BUF_PAGE_WRITTEN("system_page", "System", SYSTEM),
 
 	MONITOR_BUF_PAGE_WRITTEN("trx_system", "Transaction System",
@@ -942,57 +914,6 @@ static monitor_info_t	innodb_counter_info[] =
 	 MONITOR_EXISTING | MONITOR_DISPLAY_CURRENT | MONITOR_DEFAULT_ON),
 	 MONITOR_DEFAULT_START, MONITOR_OVLD_N_FILE_OPENED},
 
-	/* ========== Counters for Change Buffer ========== */
-	{"module_ibuf_system", "change_buffer", "InnoDB Change Buffer",
-	 MONITOR_MODULE,
-	 MONITOR_DEFAULT_START, MONITOR_MODULE_IBUF_SYSTEM},
-
-	{"ibuf_merges_insert", "change_buffer",
-	 "Number of inserted records merged by change buffering",
-	 static_cast<monitor_type_t>(
-	 MONITOR_EXISTING | MONITOR_DEFAULT_ON),
-	 MONITOR_DEFAULT_START, MONITOR_OVLD_IBUF_MERGE_INSERT},
-
-	{"ibuf_merges_delete_mark", "change_buffer",
-	 "Number of deleted records merged by change buffering",
-	 static_cast<monitor_type_t>(
-	 MONITOR_EXISTING | MONITOR_DEFAULT_ON),
-	 MONITOR_DEFAULT_START, MONITOR_OVLD_IBUF_MERGE_DELETE},
-
-	{"ibuf_merges_delete", "change_buffer",
-	 "Number of purge records merged by change buffering",
-	 static_cast<monitor_type_t>(
-	 MONITOR_EXISTING | MONITOR_DEFAULT_ON),
-	 MONITOR_DEFAULT_START, MONITOR_OVLD_IBUF_MERGE_PURGE},
-
-	{"ibuf_merges_discard_insert", "change_buffer",
-	 "Number of insert merged operations discarded",
-	 static_cast<monitor_type_t>(
-	 MONITOR_EXISTING | MONITOR_DEFAULT_ON),
-	 MONITOR_DEFAULT_START, MONITOR_OVLD_IBUF_MERGE_DISCARD_INSERT},
-
-	{"ibuf_merges_discard_delete_mark", "change_buffer",
-	 "Number of deleted merged operations discarded",
-	 static_cast<monitor_type_t>(
-	 MONITOR_EXISTING | MONITOR_DEFAULT_ON),
-	 MONITOR_DEFAULT_START, MONITOR_OVLD_IBUF_MERGE_DISCARD_DELETE},
-
-	{"ibuf_merges_discard_delete", "change_buffer",
-	 "Number of purge merged  operations discarded",
-	 static_cast<monitor_type_t>(
-	 MONITOR_EXISTING | MONITOR_DEFAULT_ON),
-	 MONITOR_DEFAULT_START, MONITOR_OVLD_IBUF_MERGE_DISCARD_PURGE},
-
-	{"ibuf_merges", "change_buffer", "Number of change buffer merges",
-	 static_cast<monitor_type_t>(
-	 MONITOR_EXISTING | MONITOR_DEFAULT_ON),
-	 MONITOR_DEFAULT_START, MONITOR_OVLD_IBUF_MERGES},
-
-	{"ibuf_size", "change_buffer", "Change buffer size in pages",
-	 static_cast<monitor_type_t>(
-	 MONITOR_EXISTING | MONITOR_DEFAULT_ON),
-	 MONITOR_DEFAULT_START, MONITOR_OVLD_IBUF_SIZE},
-
 	/* ========== Counters for server operations ========== */
 	{"module_innodb", "innodb",
 	 "Counter for general InnoDB server wide operations and properties",
@@ -1272,18 +1193,6 @@ srv_mon_set_module_control(
 	}
 }
 
-/** Reset all values.
-@param monitor  monitor identifier */
-void srv_mon_reset_all(monitor_id_t monitor) noexcept
-{
-  if (MONITOR_IS_ON(monitor))
-    sql_print_warning("InnoDB: Cannot reset all values for monitor counter '%s' "
-                      "while it is on. Please turn it off and retry.",
-                      srv_mon_get_name(monitor));
-  else
-    MONITOR_RESET_ALL(monitor);
-}
-
 /****************************************************************//**
 Get transaction system's rollback segment size in pages
 @return size in pages */
@@ -1379,13 +1288,12 @@ srv_mon_process_existing_counter(
 
 	/* innodb_buffer_pool_pages_total */
 	case MONITOR_OVLD_BUF_POOL_PAGE_TOTAL:
-	case MONITOR_OVLD_BUFFER_POOL_SIZE:
-		value = buf_pool.curr_size();
+		value = buf_pool.get_n_pages();
 		break;
 
 	/* innodb_buffer_pool_pages_misc */
 	case MONITOR_OVLD_BUF_POOL_PAGE_MISC:
-		value = buf_pool.curr_size()
+		value = buf_pool.get_n_pages()
 			- UT_LIST_GET_LEN(buf_pool.LRU)
 			- UT_LIST_GET_LEN(buf_pool.free);
 		break;
@@ -1467,7 +1375,7 @@ srv_mon_process_existing_counter(
 
 	/* innodb_os_log_written */
 	case MONITOR_OVLD_OS_LOG_WRITTEN:
-		value = log_get_lsn() - recv_sys.lsn;
+		value = log_sys.get_lsn() - recv_sys.lsn;
 		break;
 
 	/* innodb_log_waits */
@@ -1504,6 +1412,10 @@ srv_mon_process_existing_counter(
 		value = srv_page_size;
 		break;
 
+	case MONITOR_OVLD_BUFFER_POOL_SIZE:
+		value = srv_buf_pool_size;
+		break;
+
 	/* innodb_row_lock_current_waits */
 	case MONITOR_OVLD_ROW_LOCK_CURRENT_WAIT:
 		// dirty read without lock_sys.wait_mutex
@@ -1559,38 +1471,6 @@ srv_mon_process_existing_counter(
 		value = fil_system.n_open;
 		break;
 
-	case MONITOR_OVLD_IBUF_MERGE_INSERT:
-		value = ibuf.n_merged_ops[IBUF_OP_INSERT];
-		break;
-
-	case MONITOR_OVLD_IBUF_MERGE_DELETE:
-		value = ibuf.n_merged_ops[IBUF_OP_DELETE_MARK];
-		break;
-
-	case MONITOR_OVLD_IBUF_MERGE_PURGE:
-		value = ibuf.n_merged_ops[IBUF_OP_DELETE];
-		break;
-
-	case MONITOR_OVLD_IBUF_MERGE_DISCARD_INSERT:
-		value = ibuf.n_discarded_ops[IBUF_OP_INSERT];
-		break;
-
-	case MONITOR_OVLD_IBUF_MERGE_DISCARD_DELETE:
-		value = ibuf.n_discarded_ops[IBUF_OP_DELETE_MARK];
-		break;
-
-	case MONITOR_OVLD_IBUF_MERGE_DISCARD_PURGE:
-		value = ibuf.n_discarded_ops[IBUF_OP_DELETE];
-		break;
-
-	case MONITOR_OVLD_IBUF_MERGES:
-		value = ibuf.n_merges;
-		break;
-
-	case MONITOR_OVLD_IBUF_SIZE:
-		value = ibuf.size;
-		break;
-
 	case MONITOR_OVLD_SERVER_ACTIVITY:
 		value = srv_get_activity_count();
 		break;
@@ -1600,7 +1480,7 @@ srv_mon_process_existing_counter(
 		break;
 
 	case MONITOR_OVLD_LSN_CURRENT:
-		value = log_get_lsn();
+		value = log_sys.get_lsn();
 		break;
 
         case MONITOR_OVLD_CHECKPOINTS:
@@ -1608,10 +1488,10 @@ srv_mon_process_existing_counter(
 		break;
 
 	case MONITOR_LSN_CHECKPOINT_AGE:
-		log_sys.latch.wr_lock(SRW_LOCK_CALL);
+		log_sys.latch.rd_lock(SRW_LOCK_CALL);
 		value = static_cast<mon_type_t>(log_sys.get_lsn()
 						- log_sys.last_checkpoint_lsn);
-		log_sys.latch.wr_unlock();
+		log_sys.latch.rd_unlock();
 		break;
 
 	case MONITOR_OVLD_BUF_OLDEST_LSN:
