@@ -12,7 +12,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
 
 #define MYSQL_SERVER 1
 #include <my_global.h>
@@ -986,7 +986,7 @@ spider_group_by_handler::spider_group_by_handler(
   DBUG_ENTER("spider_group_by_handler::spider_group_by_handler");
   spider = fields->get_first_table_holder()->spider;
   trx = spider->wide_handler->trx;
-  my_bitmap_init(&skips, NULL, skips1.n_bits, TRUE);
+  my_bitmap_init(&skips, NULL, skips1.n_bits);
   bitmap_copy(&skips, &skips1);
   DBUG_VOID_RETURN;
 }
@@ -1010,7 +1010,6 @@ static int spider_prepare_init_scan(
   const Query& query, MY_BITMAP *skips, spider_fields *fields, ha_spider *spider,
   SPIDER_TRX *trx, longlong& offset_limit, THD *thd)
 {
-  int error_num, link_idx;
   SPIDER_RESULT_LIST *result_list = &spider->result_list;
   st_select_lex *select_lex;
   longlong select_limit, direct_order_limit;
@@ -1037,10 +1036,6 @@ static int spider_prepare_init_scan(
 
   spider_db_free_one_result_for_start_next(spider);
 
-  spider->sql_kinds = SPIDER_SQL_KIND_SQL;
-  for (link_idx = 0; link_idx < (int) share->link_count; ++link_idx)
-    spider->sql_kind[link_idx] = SPIDER_SQL_KIND_SQL;
-
   spider->do_direct_update = FALSE;
   spider->direct_update_kinds = 0;
   spider_get_select_limit(spider, &select_lex, &select_limit, &offset_limit);
@@ -1054,9 +1049,7 @@ static int spider_prepare_init_scan(
   ) {
     result_list->internal_limit = select_limit /* + offset_limit */;
     result_list->split_read = select_limit /* + offset_limit */;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
     result_list->bgs_split_read = select_limit /* + offset_limit */;
-#endif
 
     result_list->split_read_base = 9223372036854775807LL;
     result_list->semi_split_read = 0;
@@ -1067,10 +1060,8 @@ static int spider_prepare_init_scan(
   }
   result_list->semi_split_read_base = 0;
   result_list->set_split_read = TRUE;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
-  if ((error_num = spider_set_conn_bg_param(spider)))
+  if (int error_num = spider_set_conn_bg_param(spider))
     DBUG_RETURN(error_num);
-#endif
   DBUG_PRINT("info",("spider result_list.finish_flg = FALSE"));
   result_list->finish_flg = FALSE;
   result_list->record_num = 0;
@@ -1182,7 +1173,6 @@ static int spider_send_query(
     link_idx = link_idx_holder->link_idx;
     dbton_hdl = spider->dbton_handler[conn->dbton_id];
     spider->link_idx_chain = link_idx_chain;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
     if (result_list->bgs_phase > 0)
     {
       if ((error_num = spider_check_and_init_casual_read(trx->thd, spider,
@@ -1201,8 +1191,8 @@ static int spider_send_query(
         }
         DBUG_RETURN(error_num);
       }
-    } else {
-#endif
+    } else
+    {
       if ((error_num = dbton_hdl->set_sql_for_exec(
              SPIDER_SQL_TYPE_SELECT_SQL, link_idx, link_idx_chain)))
         DBUG_RETURN(error_num);
@@ -1262,9 +1252,7 @@ static int spider_send_query(
         spider_db_discard_result(spider, link_idx, conn);
         spider_unlock_after_query(conn, 0);
       }
-#ifndef WITHOUT_SPIDER_BG_SEARCH
     }
-#endif
   }
   DBUG_RETURN(0);
 }
@@ -1334,7 +1322,6 @@ int spider_group_by_handler::next_row()
           table->status = STATUS_NOT_FOUND;
         DBUG_RETURN(spider->store_error_num);
       }
-#ifndef WITHOUT_SPIDER_BG_SEARCH
       if (spider->result_list.bgs_phase > 0)
       {
         fields->set_pos_to_first_link_idx_chain();
@@ -1363,7 +1350,6 @@ int spider_group_by_handler::next_row()
           }
         }
       }
-#endif
       spider->use_pre_call = FALSE;
     }
   } else if (offset_limit)
@@ -1413,6 +1399,9 @@ group_by_handler *spider_create_group_by_handler(
   MY_BITMAP skips;
   DBUG_ENTER("spider_create_group_by_handler");
 
+  if (spider_param_disable_group_by_handler(thd))
+    DBUG_RETURN(NULL);
+
   switch (thd_sql_command(thd))
   {
     case SQLCOM_UPDATE:
@@ -1429,7 +1418,6 @@ group_by_handler *spider_create_group_by_handler(
   do {
     DBUG_PRINT("info",("spider from=%p", from));
     ++table_count;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       DBUG_PRINT("info",("spider partition handler"));
@@ -1442,16 +1430,14 @@ group_by_handler *spider_create_group_by_handler(
         DBUG_RETURN(NULL);
       }
     }
-#endif
   } while ((from = from->next_local));
 
   if (!(table_holder= spider_create_table_holder(table_count)))
     DBUG_RETURN(NULL);
 
-  my_bitmap_init(&skips, NULL, query->select->elements, TRUE);
+  my_bitmap_init(&skips, NULL, query->select->elements);
   table_idx = 0;
   from = query->from;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   if (from->table->part_info)
   {
     partition_info *part_info = from->table->part_info;
@@ -1462,7 +1448,6 @@ group_by_handler *spider_create_group_by_handler(
   } else {
     spider = (ha_spider *) from->table->file;
   }
-#endif
   share = spider->share;
   spider->idx_for_direct_join = table_idx;
   ++table_idx;
@@ -1484,7 +1469,6 @@ group_by_handler *spider_create_group_by_handler(
   }
   while ((from = from->next_local))
   {
-#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1495,7 +1479,6 @@ group_by_handler *spider_create_group_by_handler(
     } else {
       spider = (ha_spider *) from->table->file;
     }
-#endif
     share = spider->share;
     spider->idx_for_direct_join = table_idx;
     ++table_idx;
@@ -1524,7 +1507,6 @@ group_by_handler *spider_create_group_by_handler(
 
   from = query->from;
   do {
-#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1535,7 +1517,6 @@ group_by_handler *spider_create_group_by_handler(
     } else {
       spider = (ha_spider *) from->table->file;
     }
-#endif
     share = spider->share;
     if (spider_param_skip_default_condition(thd,
       share->skip_default_condition))
@@ -1566,16 +1547,6 @@ group_by_handler *spider_create_group_by_handler(
         DBUG_PRINT("info",("spider select item=%p", item));
         if (item->const_item())
         {
-          /*
-            Do not create the GBH when a derived table or view is
-            involved
-          */
-          if (thd->derived_tables != NULL)
-          {
-            keep_going= FALSE;
-            break;
-          }
-
           /*
             Do not handle the complex case where there's a const item
             in the auxiliary fields. It is too unlikely (if at all) to
@@ -1693,7 +1664,6 @@ group_by_handler *spider_create_group_by_handler(
     goto skip_free_table_holder;
 
   from = query->from;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   if (from->table->part_info)
   {
     partition_info *part_info = from->table->part_info;
@@ -1704,7 +1674,6 @@ group_by_handler *spider_create_group_by_handler(
   } else {
     spider = (ha_spider *) from->table->file;
   }
-#endif
   share = spider->share;
   lock_mode = spider_conn_lock_mode(spider);
   if (lock_mode)
@@ -1729,13 +1698,6 @@ group_by_handler *spider_create_group_by_handler(
       spider->conn_link_idx, roop_count, share->link_count,
       tgt_link_status)
   ) {
-    if (spider_param_use_handler(thd, share->use_handlers[roop_count]))
-    {
-      DBUG_PRINT("info",("spider direct_join does not support use_handler"));
-      if (lock_mode)
-        goto skip_free_fields;
-      continue;
-    }
     conn = spider->conns[roop_count];
     DBUG_PRINT("info",("spider roop_count=%d", roop_count));
     DBUG_PRINT("info",("spider conn=%p", conn));
@@ -1770,7 +1732,6 @@ group_by_handler *spider_create_group_by_handler(
   {
     fields->clear_conn_holder_from_conn();
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
     if (from->table->part_info)
     {
       partition_info *part_info = from->table->part_info;
@@ -1781,7 +1742,6 @@ group_by_handler *spider_create_group_by_handler(
     } else {
       spider = (ha_spider *) from->table->file;
     }
-#endif
     share = spider->share;
     DBUG_PRINT("info",("spider s->db=%s", from->table->s->db.str));
     DBUG_PRINT("info",("spider s->table_name=%s", from->table->s->table_name.str));
@@ -1800,13 +1760,6 @@ group_by_handler *spider_create_group_by_handler(
         tgt_link_status)
     ) {
       DBUG_PRINT("info",("spider roop_count=%d", roop_count));
-      if (spider_param_use_handler(thd, share->use_handlers[roop_count]))
-      {
-        DBUG_PRINT("info",("spider direct_join does not support use_handler"));
-        if (lock_mode)
-          goto skip_free_fields;
-        continue;
-      }
       conn = spider->conns[roop_count];
       DBUG_PRINT("info",("spider conn=%p", conn));
       if (!fields->check_conn_same_conn(conn))
